# -*- coding: utf-8 -*-

import os
from django.test import TestCase
from django.conf import settings

from django_stachoutils import lowerize, NO_CAPFIRST


class UtilsTestCase(TestCase):
    def test_files_are_equal(self):
        from django.core.files.base import ContentFile
        from django_stachoutils import files_are_equal
        MEDIA_ROOT = os.path.join(settings.MEDIA_ROOT, 'test')
        img_root = os.path.join(MEDIA_ROOT, 'images')

        image1 = os.path.join(img_root, 'jardin.jpg')
        image2 = os.path.join(img_root, 'terrasse.jpg')
        f1 = ContentFile(open(image1, 'rb').read())
        f2 = ContentFile(open(image2, 'rb').read())

        self.assertFalse(files_are_equal(f1, f2))

        f1bis = ContentFile(open(image1, 'rb').read())
        self.assertTrue(files_are_equal(f1, f1bis))

    def test_lowerize(self):
        for test_string, expected in [
                    ('LOWER THIS', 'Lower This'),
                    ('Les Carrés De Monthoux', 'Les Carrés de Monthoux'),
                ]:
            self.assertEqual(lowerize(test_string, NO_CAPFIRST), expected)



# -*- coding: utf-8 -*-

from django import forms
from django.forms.widgets import flatatt
from django.utils.html import conditional_escape
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from django.contrib.admin import widgets as admin_widgets


BUTTONS = [{'class': 'gras', 'text': u'Gras', 'accesskey': 'g'},
           {'class': 'sup', 'text': u'Supérieur', 'accesskey': 's'},
           {'class': 'bas', 'text': u'Bas de casse', 'accesskey': 'b'}]

STATICS = {'js': ['django_stachoutils/js/jquery.charcounter.js',
                  'django_stachoutils/js/jquery.fieldselection.js',
                  'django_stachoutils/js/widgets.js']}


class TexteEditeur(forms.Textarea):
    def render(self, name, value, attrs=None):
        if value is None: value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        html = []
        html.append(u'<div class="editeur"><div class="toolbar">')
        for button in BUTTONS:
            html.append(u'<button type="button" accesskey="%(accesskey)s" class="%(class)s">%(text)s</button>' % button)
        html.append(u'</div>')
        html.append(u'<textarea%s>%s</textarea>' % (flatatt(final_attrs),
                                                    conditional_escape(force_unicode(value))))
        html.append(u'</div>')
        return mark_safe(u'\n'.join(html))

    def _media(self):
        return forms.Media(**STATICS)
    media = property(_media)


class AdminTexteEditeur(admin_widgets.AdminTextareaWidget, TexteEditeur):
    pass


class TextareaCounter(AdminTexteEditeur):
    def __init__(self, max_signs=None, direction='up', attrs=None):
        self.max_signs = max_signs
        self.direction = direction
        super(TextareaCounter, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        textarea = super(TextareaCounter, self).render(name, value, attrs)
        jscounter = ''
        if self.max_signs:
            label = "signe(s)"
            if self.direction == 'down':
                label += ' restant(s)'
            jscounter = (
                '<script type="text/javascript">'
                    '$(document).ready(function () { '
                        '$("#%(id)s").charCounter(%(max_signs)d, {'
                              'container: "<div></div>",'
                              'direction: "%(direction)s",'
                              'format: "%%1 %(label)s",'
                              'limit: false,'
                              'limit_class: "error"'
                        '});'
                    '});'
                '</script>' % {
                    'id': attrs['id'], 
                    'direction': self.direction,
                    'label': label,
                    'max_signs': self.max_signs})
        return mark_safe(textarea + jscounter)


class TextInputCounter(forms.TextInput):
    def __init__(self, max_signs=None, direction='up', attrs=None):
        self.max_signs = max_signs
        self.direction = direction
        super(TextInputCounter, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        textinput = super(TextInputCounter, self).render(name, value, attrs)
        jscounter = ''
        if self.max_signs:
            label = "signe(s)"
            if self.direction == 'down':
                label += ' restant(s)'
            jscounter = (
                '<script type="text/javascript">'
                    '$(document).ready(function () { '
                        '$("#%(id)s").charCounter(%(max_signs)d, {'
                              'container: "<span></span>",'
                              'direction: "%(direction)s",'
                              'format: "%%1 %(label)s",'
                              'limit: false,'
                              'limit_class: "error"'
                        '});'
                    '});'
                '</script>' % {
                    'id': attrs['id'], 
                    'direction': self.direction,
                    'label': label,
                    'max_signs': self.max_signs})
        return mark_safe(textinput + jscounter)

    def _media(self):
        return forms.Media(**STATICS)
    media = property(_media)



# -*- coding: utf-8 -*-

from datetime import date
from django.core.management.base import BaseCommand
from optparse import make_option


class Command(BaseCommand):
    help = u"Load a database dump."

    option_list = BaseCommand.option_list + (
        make_option('--source',
                    action='store',
                    type="string",
                    dest='source',
                    help=u"Path to the database dump file to load."),
        make_option('--destination',
                    action='store',
                    type="string",
                    dest='destination',
                    default='local',
                    help=u"Where to load the dump."),
        make_option('--dry-run',
                    action='store_true',
                    dest='dry_run',
                    default=False,
                    help=u'Just pretend...'),
    )

    def handle(self, *args, **options):
        command.stdout.write("Loading %s on %s" % (options.get("source"), options.get("destination")))



# -*- coding: utf-8 -*-

from django import forms
from django.db import models
from django.conf.urls import patterns
from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.core import serializers
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext


class DumpdataForm(forms.Form):
    content_type = forms.CharField(widget=forms.HiddenInput, required=True)
    selected = forms.CharField(widget=forms.HiddenInput, required=True)
    use_natural_keys = forms.BooleanField(label="Use natural Keys", initial=True, required=False)
    dump_format = forms.ChoiceField(choices=[('json', 'Json'), ('xml', 'XML')], initial='json', required=True)
    indent_level = forms.IntegerField(min_value=0, max_value=8, initial=2, widget=forms.TextInput(attrs={"size": "2"}), required=True)


class ModelAdmin(admin.ModelAdmin):
    actions = ["dumpdata"]

    def get_actions(self, request):
        actions = super(ModelAdmin, self).get_actions(request)
        if not request.user.is_superuser and "dumpdata" in actions:
            del actions["dumpdata"]
        return actions

    def get_urls(self):
        return patterns('',
            (r'^dumpdata/$', self.admin_site.admin_view(self.dumpdata_form)),
        ) + super(ModelAdmin, self).get_urls()

    def dumpdata(self, request, queryset):
        selected = request.POST.getlist(admin.ACTION_CHECKBOX_NAME)
        ct = ContentType.objects.get_for_model(queryset.model)
        return HttpResponseRedirect("dumpdata/?ct=%s&ids=%s" % (ct.pk, ",".join(selected)))

    def dumpdata_form(self, request):
        if request.method == "GET":
            initial = {
                'content_type': request.GET["ct"],
                'selected': request.GET["ids"]
            }
            form = DumpdataForm(None, initial=initial)
        elif request.method == "POST":
            form = DumpdataForm(request.POST)
            if form.is_valid():
                json = {}
                fmt = form.cleaned_data['dump_format']
                indent = form.cleaned_data['indent_level']
                use_natural_keys = form.cleaned_data['use_natural_keys']

                ctype = ContentType.objects.get_for_id(form.cleaned_data["content_type"])
                queryset = ctype.get_all_objects_for_this_type().filter(
                    pk__in=form.cleaned_data["selected"].split(","))

                objects = list(queryset)
                dependancies = []
                for obj in queryset:
                    append_obj_dependancies(obj, dependancies) 
                objects = dependancies + objects
                #TODO sort object by appname.modelname.
                # use_natural_keys fout la merde dans les dossiers pour les ref user.

                stream = serializers.serialize(fmt, objects, indent=indent,
                                               use_natural_keys=use_natural_keys)
                return HttpResponse(stream, content_type='application/json')
        return render_to_response('django_stachoutils/admin/dumpdata_form.html', {
            'title': "Dump data",
            'is_popup': request.REQUEST.has_key('_popup'),
            'form': form,
        }, context_instance=RequestContext(request))
    dumpdata.short_description = u'Dump the selected objects'


def append_obj_dependancies(obj, dependancies, add_m2m=True):
    for field in obj.__class__._meta.fields:
        if isinstance(field, models.ForeignKey):
            rel = getattr(obj, field.name)
            if rel and rel not in dependancies:
                dependancies.insert(0, rel)
                append_obj_dependancies(rel, dependancies, add_m2m=False)
    if add_m2m:
        for related in obj._meta.get_all_related_objects():
            try:
                objs = getattr(obj, related.get_accessor_name()).all()
            # Cas des OneToOne.
            except AttributeError:
                objs = [getattr(obj, related.field.related_query_name())]
            for rel in objs:
                dependancies.insert(0, rel)
                append_obj_dependancies(rel, dependancies, add_m2m=False)



# -*- coding: utf-8 -*-

import unittest


class StachoutilsTestCase(unittest.TestCase):
    def test_lowerize(self):
        from django_stachoutils import lowerize

        self.assertEqual(lowerize(u"L'ARBRE à CHAT"), u"L'Arbre à Chat")


def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(StachoutilsTestCase)
    return suite



# -*- coding: utf-8 -*-

import logging
import logging.handlers
import string
from django.core import mail
from django.conf import settings
from django.contrib import messages


# Python < 2.7.
class NullHandler(logging.Handler):
    def emit(self, record):
        pass


# http://www.red-dove.com/python_logging.html
class BufferingSMTPHandler(logging.handlers.BufferingHandler):
    def __init__(self, mailhost, fromaddr, toaddrs, subject, capacity):
        logging.handlers.BufferingHandler.__init__(self, capacity)
        self.mailhost = mailhost
        self.mailport = None
        self.fromaddr = fromaddr
        self.toaddrs = toaddrs
        self.subject = subject
        self.setFormatter(logging.Formatter("%(asctime)s %(levelname)-5s %(message)s"))

    def flush(self):
        if len(self.buffer) > 0:
            try:
                import smtplib
                port = self.mailport
                if not port:
                    port = smtplib.SMTP_PORT
                smtp = smtplib.SMTP(self.mailhost, port)
                msg = "From: %s\r\nTo: %s\r\nSubject: %s\r\n\r\n" % (self.fromaddr, string.join(self.toaddrs, ","), self.subject)
                for record in self.buffer:
                    s = self.format(record)
                    print s
                    msg = msg + s + "\r\n"
                smtp.sendmail(self.fromaddr, self.toaddrs, msg)
                smtp.quit()
            except:
                self.handleError(None)  # no particular record
            self.buffer = []


class BaseEmailHandler(logging.Handler):
    """Beneficie de la conf SMTP dans settings par rapport à logging.handlers.SMTPHandler. """
    def __init__(self, subject=""):
        """Allow one to set the subject in settings.LOGGING DictConfig. """
        logging.Handler.__init__(self)
        self.subject = subject

    def emit(self, record):
        subject = u'%s[%s] : %s' % (settings.EMAIL_SUBJECT_PREFIX, record.levelname, self.subject)
        message = record.msg
        mail.send_mail(subject, message, settings.DEFAULT_FROM_EMAIL, self.recipient_list, fail_silently=True)


class AdminEmailHandler(BaseEmailHandler):
    def emit(self, record):
        self.recipient_list = [email for name, email in settings.ADMINS]
        BaseEmailHandler.emit(self, record)


levelname_to_int = {
    'DEBUG': messages.DEBUG,
    'INFO': messages.INFO,
    'WARNING': messages.WARNING,
    'ERROR': messages.ERROR,
}


class MessagelHandler(logging.Handler):
    """An handler proxiing to messages. """

    def __init__(self):
        logging.Handler.__init__(self)

    def emit(self, record):
        messages.add_message(record.request, levelname_to_int[record.levelname], record.getMessage())



# -*- coding: utf-8 -*-

from django import VERSION as DJ_VERSION

if DJ_VERSION < (1, 8):
    from django.db import models

    # http://djangosnippets.org/snippets/2100/
    # https://code.djangoproject.com/ticket/11305

    class SQLCountCase(models.sql.aggregates.Aggregate):
        is_ordinal = True
        sql_function = 'COUNT'
        sql_template = "%(function)s(CASE %(case)s WHEN %(when)s THEN 1 ELSE null END)"

        def __init__(self, col, **extra):
            if isinstance(extra['when'], basestring):
                quote = extra.get('quote', "'")
                extra['when'] = "(%s%s%s)" % (quote, extra['when'], quote)

            if not extra.get('case', None) and not extra.has_key('case'):
                extra['case'] = '"%s"."%s"' % (extra['source'].model._meta.db_table, extra['source'].name)

            if extra['when'] is None:
                extra['when'] = True
                extra['case'] += ' IS NULL '

            super(SQLCountCase, self).__init__(col, **extra)


    class CountCase(models.Aggregate):
        name = 'COUNT'

        def add_to_query(self, query, alias, col, source, is_summary):
            aggregate = SQLCountCase(col, source=source, is_summary=is_summary, **self.extra)
            query.aggregates[alias] = aggregate


    class SQLSumExtra(models.sql.aggregates.Aggregate):
        is_ordinal = False
        sql_function = 'SUM'
        sql_template = "%(function)s(%(sum)s)"

        def __init__(self, col, **extra):
            super(SQLSumExtra, self).__init__(col, **extra)


    class SumExtra(models.Aggregate):
        name = 'SUM'

        def add_to_query(self, query, alias, col, source, is_summary):
            aggregate = SQLSumExtra(col, source=source, is_summary=is_summary, **self.extra)
            query.aggregates[alias] = aggregate


    #http://djangosnippets.org/snippets/2099/
    class SQLSumCase(models.sql.aggregates.Aggregate):
        is_ordinal = True
        sql_function = 'SUM'
        sql_template = "%(function)s(CASE %(case)s WHEN %(when)s THEN %(field)s ELSE 0 END)"

        def __init__(self, col, **extra):
            if isinstance(extra['when'], basestring):
                extra['when'] = "'%s'" % extra['when']

            if not extra.get('case', None):
                extra['case'] = '"%s"."%s"' % (extra['source'].model._meta.db_table, extra['source'].name)
            # FIXME: gerer les relation via double underscore.
            else:
                extra['case'] = '"%s"."%s"' % (extra['source'].model._meta.db_table, extra['case'])

            if extra['when'] is None:
                extra['when'] = True
                extra['case'] += ' IS NULL '

            super(SQLSumCase, self).__init__(col, **extra)


    class SumCase(models.Aggregate): # TODO
        name = 'SUM'

        def add_to_query(self, query, alias, col, source, is_summary):
            aggregate = SQLSumCase(col, source=source, is_summary=is_summary, **self.extra)
            query.aggregates[alias] = aggregate
else:
    from django.db.models import Aggregate, IntegerField


    class SumExtra(Aggregate):
        function = 'SUM'
        sql_template = "%(function)s(%(sum_expressions)s)"


    class SumCase(Aggregate):
        function = 'SUM'
        template = "%(function)s(CASE %(expressions)s WHEN %(when)s THEN %(expressions)s ELSE 0 END)"

        def __init__(self, expression, **extra):
            if isinstance(extra['when'], basestring):
                extra['when'] = "'%s'" % extra['when']
            # Pas de cas d'utilisation. A vérifier car réécriture pour Django-1.8.
            if extra['when'] is None:
                extra['when'] = True
                extra['case'] += ' IS NULL '

            super(SumCase, self).__init__(
                expression,
                output_field=IntegerField(),
                **extra)


    class CountCase(Aggregate):
        function = 'Count'
        template = "%(function)s(CASE %(expressions)s WHEN %(when)s THEN 1 ELSE null END)"

        def __init__(self, expression, **extra):
            if isinstance(extra['when'], basestring):
                quote = extra.get('quote', "'")
                extra['when'] = "(%s%s%s)" % (quote, extra['when'], quote)
            # Pas de cas d'utilisation. A vérifier car réécriture pour Django-1.8.
            if extra['when'] is None:
                extra['when'] = True
                extra['case'] += ' IS NULL '
            super(CountCase, self).__init__(
                expression,
                output_field=IntegerField(),
                **extra)


    class CountCase2(Aggregate):
        function = 'Count'
        template = "%(function)s(CASE WHEN %(when)s THEN 1 ELSE null END)"

        def __init__(self, expression, **extra):
            if isinstance(extra['when'], basestring):
                quote = extra.get('quote', "'")
                extra['when'] = "(%s%s%s)" % (quote, extra['when'], quote)
            super(CountCase2, self).__init__(
                expression,
                output_field=IntegerField(),
                **extra)



from django.contrib import admin
from django.contrib.admin.templatetags.admin_static import static
from django_stachoutils.admin import ModelAdmin
from my_app import models


class MyAdmin(admin.AdminSite):
    pass


class BandMemberAdmin(ModelAdmin):
    pass
    

admin_site = MyAdmin(name='my-admin')
admin_site.register(models.BandMember, BandMemberAdmin)



# -*- coding: utf-8 -*-

import os
import unittest



class TemplateTagsTestCase(unittest.TestCase):
    def test_current_filters_html(self):
        os.environ['DJANGO_SETTINGS_MODULE'] = "settings"
        from django_stachoutils.templatetags.stachoutils_extras import current_filters
        filters = [
            (u'statut', False, [
                ('valide__exact=1&assistant__exact=6', u'Tout', True),
                ('valide__exact=1&assistant__exact=6&statut__exact=1', u'Relation Client', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=2', u'Production', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=3', u'P.A.O.', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=4', u'Publicit\xe9 \xe0 Valider', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=5', u'Accord Bon \xe0 Tirer', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=6', u'Publicit\xe9 Modifi\xe9e', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=7', u'OK Fab', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=8', u'Correction Demand\xe9es', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=9', u'BAT Bloqu\xe9', False)
            ]),
            (u'BAT demand\xe9', False, [
                ('valide__exact=1&assistant__exact=6', u'Tout', None),
                ('valide__exact=1&assistant__exact=6&bat_demande__exact=1', 'Oui', False),
                ('valide__exact=1&assistant__exact=6&bat_demande__exact=0', 'Non', True)
            ]),
            (u'charg\xe9 de client\xe8le', False, [
                ('valide__exact=1', u'Tout', None),
                ('valide__exact=1&assistant__exact=8', u'Isabelle G.', False),
                ('valide__exact=1&assistant__exact=6', u'Marie-Fran\xe7oise B.', True),
                ('valide__exact=1&assistant__exact=41', u'\xc9milie G.', False),
                ('valide__exact=1&assistant__exact=1445', u'S\xe9verine T.', False),
                ('assistant__isnull=True&valide__exact=1&assistant__exact=6', u'Aucun', None)
            ]),
        ]

        self.assertEqual(current_filters(filters),
                         [u'BAT demand\xe9: <strong>Non</strong>',
                          u'charg\xe9 de client\xe8le: <strong>Marie-Fran\xe7oise B.</strong>'])
        
        # Should works with a filter not selected.
        filters = [
            (u'statut', False, [
                ('valide__exact=1&assistant__exact=6', u'Tout', None),
                ('valide__exact=1&assistant__exact=6&statut__exact=1', u'Relation Client', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=2', u'Production', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=3', u'P.A.O.', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=4', u'Publicit\xe9 \xe0 Valider', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=5', u'Accord Bon \xe0 Tirer', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=6', u'Publicit\xe9 Modifi\xe9e', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=7', u'OK Fab', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=8', u'Correction Demand\xe9es', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=9', u'BAT Bloqu\xe9', False)
            ]),
            (u'BAT demand\xe9', False, [
                ('valide__exact=1&assistant__exact=6', u'Tout', None),
                ('valide__exact=1&assistant__exact=6&bat_demande__exact=1', 'Oui', False),
                ('valide__exact=1&assistant__exact=6&bat_demande__exact=0', 'Non', True)
            ]),
            (u'charg\xe9 de client\xe8le', False, [
                ('valide__exact=1', u'Tout', None),
                ('valide__exact=1&assistant__exact=8', u'Isabelle G.', False),
                ('valide__exact=1&assistant__exact=6', u'Marie-Fran\xe7oise B.', True),
                ('valide__exact=1&assistant__exact=41', u'\xc9milie G.', False),
                ('valide__exact=1&assistant__exact=1445', u'S\xe9verine T.', False),
                ('assistant__isnull=True&valide__exact=1&assistant__exact=6', u'Aucun', None)
            ]),
        ]

        self.assertEqual(current_filters(filters),
                         [u'BAT demand\xe9: <strong>Non</strong>',
                          u'charg\xe9 de client\xe8le: <strong>Marie-Fran\xe7oise B.</strong>'])



def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(TemplateTagsTestCase)
    return suite



# -*- coding: utf-8 -*-

from django.shortcuts import get_object_or_404

from django import forms
from django.conf import settings
from django.utils.safestring import mark_safe

from django.forms.models import ModelFormMetaclass


NESTED_NON_FIELD_ERRORS = '__nested__'


class NestedModelFormOptions(object):
    def __init__(self, options=None):
        self.Form = getattr(options, 'form', None)  # Class.
        self.fk = getattr(options, 'fk', None)  # fk name.


class NestedModelFormMetaclass(ModelFormMetaclass):
    """ Ajoute les attributs self._nested.Form et self._nested.fk"""
    def __new__(cls, name, bases, attrs):
        new_class = super(NestedModelFormMetaclass, cls).__new__(cls, name, bases, attrs)
        new_class._nested = NestedModelFormOptions(getattr(new_class, 'Nested', None))  # Declarez une class Nested.
        return new_class


class NestedModelForm(forms.ModelForm):
    __metaclass__ = NestedModelFormMetaclass

    def __init__(self, *args, **kwargs):
        self.forms = []
        self.nested_form = None
        self._non_nested_errors = None
        nested_initial, nested_instance = {}, None
        if ('initial' in kwargs) and ('NESTED' in kwargs['initial']):
            nested_initial = kwargs['initial'].pop('NESTED')
        super(NestedModelForm, self).__init__(*args, **kwargs)
        nested_id = self.initial.get(self._nested.fk, None)  # self.initial est un merge du paramètre 'initial' et des  attrs de l'instance.
        if '%s-%s' % (self.prefix, self._nested.fk) in self.data:  # Le post écrase initial si présent.
            nested_id = self.data['%s-%s' % (self.prefix, self._nested.fk)]
        if nested_id:
            nested_instance = get_object_or_404(self._nested.Form.Meta.model, pk=nested_id)
        self._init_nested_form(nested_instance, nested_initial)

    def _init_nested_form(self, instance, initial):
        prefix = '%s_%s' % (self.prefix, self._nested.fk.upper())
        self.nested_form = self._nested.Form(self.data or None, self.files or None,
                                             prefix=prefix, initial=initial, instance=instance)
        self.forms.append(self.nested_form)

    def _get_errors(self):
        if self._errors is None:
            super(NestedModelForm, self).errors
            if not self.is_bound:  # Stop further processing.
                return self._errors
            if self.has_changed() or self.nested_form.has_changed():
                self.nested_form.empty_permitted = True
            nested_errors = self.nested_form.errors
            if nested_errors:
                self._errors[NESTED_NON_FIELD_ERRORS] = self.nested_form.non_field_errors()
        return self._errors
    errors = property(_get_errors)

    def _get_non_nested_errors(self):
        if self._non_nested_errors is None:
            self._non_nested_errors = self.errors.copy()
            if NESTED_NON_FIELD_ERRORS in self._non_nested_errors:
                del self._non_nested_errors[NESTED_NON_FIELD_ERRORS]
        return self._non_nested_errors
    non_nested_errors = property(_get_non_nested_errors)

    def has_changed(self, *args, **kwargs):
        nested_has_changed = self.nested_form.has_changed()
        host_has_changed = super(NestedModelForm, self).has_changed(*args, **kwargs)
        if host_has_changed or nested_has_changed:
            return True
        self.nested_form._errors = {}  # Why do we need to do that ? If not, next
        # call to errors (i.e. from the template) will revalidate and add errors although
        # It (the nested form) has not changed.
        return False

    def save(self, *args, **kwargs):
        instance = self.instance
        self.nested_instance = None
        if self.nested_form.has_changed():
            self.nested_instance = self.nested_form.save()
        instance = super(NestedModelForm, self).save(*args, **kwargs)
        if self.nested_instance:
            setattr(instance, self._nested.fk, self.nested_instance)
            instance.save()
        return instance


# http://djangosnippets.org/snippets/2248/

class ModelFormOptions(object):
    def __init__(self, options=None):
        self.inlines = getattr(options, 'inlines', {})


class ModelFormMetaclass(ModelFormMetaclass):
    """ Ajoute les attributs self._forms.inlines """
    def __new__(cls, name, bases, attrs):
        new_class = super(ModelFormMetaclass, cls).__new__(cls, name, bases, attrs)
        new_class._forms = ModelFormOptions(getattr(new_class, 'Forms', None))
        return new_class


class ModelForm(forms.ModelForm):
    """
    Add to ModelForm the ability to declare inline formsets.

    It save you from the boiler-plate implementation of cross validation/saving of such forms
    in the views.
    You should use It in the admin's forms if you need the inherit them in your apps because
    there is not multi-inherance.

    >>> from myapp import models
    >>> class Program(models.Model):
    ...     name = models.CharField(max_length=100, blank=True)

    >>> class ImageProgram(models.Model):
    ...     image = models.ImageField('image')
    ...     program = models.ForeignKey(Program)

    >>> class Ringtone(models.Model):
    ...     sound = models.FileField('sound')
    ...     program = models.ForeignKey(Program)

    Use It in your admin.py instead of django.forms.ModelForm:
    >>> class ProgramAdminForm(ModelForm):
    ...     class Meta:
    ...         model = Program
    ...         def clean(self):
    ...             cleaned_data = self.cleaned_data
    ...             # stuff
    ...             return cleaned_data

    In your app, say you declare the following inline formsets:
    >>> from django.forms.models import inlineformset_factory
    >>> from myapp.forms import ImageProgramForm, RingtoneProgramForm
    >>> ImageProgramFormSet = inlineformset_factory(models.Program, models.ImageProgram,
    ...                                             form=ImageProgramForm, max_num=6)
    >>> RingToneFormSet = inlineformset_factory(models.Program, models.RingTone,
    ...                                         form=RingtoneProgramForm)

    You can bind them in your program's form:
    >>> class MyProgramForm(ProgramAdminForm):
    ...     class Forms:
    ...         inlines = {
    ...             'images': ImageProgramFormSet,
    ...             'ringtones': RingToneFormSet,
    ...         }

    And instanciate It:
    >>> def my_view(request):
    ...     program_form = MyProgramForm(request.POST, request.FILES, prefix='prog')

    In the template, you access the inlines like that :
    {{ program_form.inlineformsets.images.management_form }}
    {{ program_form.inlineformsets.images.non_form_errors }}
    <table>
    {{ program_form.inlineformsets.images.as_table }}
    </table>


    """
    __metaclass__ = ModelFormMetaclass

    def __init__(self, *args, **kwargs):
        super(ModelForm, self).__init__(*args, **kwargs)
        if hasattr(self._forms, 'inlines'):
            self.inlineformsets = {}
            fset_initial = kwargs.get('initial', {}).get("_INLINES", {})
            for key, FormSet in self._forms.inlines.items():
                initial = fset_initial.get(key, {})
                self.inlineformsets[key] = FormSet(self.data or None, self.files or None,
                                                   prefix=self._get_formset_prefix(key),
                                                   initial=initial, instance=self.instance)

    def save(self, *args, **kwargs):
        instance = super(ModelForm, self).save(*args, **kwargs)
        if hasattr(self._forms, 'inlines'):
            for fset in self.inlineformsets.values():
                if not fset.has_changed():
                    continue
                fset.save()
        return instance

    def has_changed(self, *args, **kwargs):
        has_changed = super(ModelForm, self).has_changed(*args, **kwargs)
        if has_changed:
            return True
        else:
            for fset in self.inlineformsets.values():
                for i in range(0, fset.total_form_count()):
                    form = fset.forms[i]
                    if form.has_changed():
                        return True
        return False

    def _get_formset_prefix(self, key):
        return u'%s_%s' % (self.prefix, key.upper())

    def _clean_form(self):
        super(ModelForm, self)._clean_form()
        for key, fset in self.inlineformsets.items():
            for i in range(0, fset.total_form_count()):
                f = fset.forms[i]
                if not f.has_changed():
                    continue
                if f.errors:
                    self._errors['_%s_%d' % (fset.prefix, i)] = f.non_field_errors  # Pourquoi non_field_errors ?

    # This should be in forms.Form.
    def as_tr(self):
        return self._html_output(
            normal_row=u'<td%(html_class_attr)s>%(field)s</td>',
            error_row=u'%s',
            row_ender='',
            help_text_html=u'',
            errors_on_separate_row=True
        )


# Image Fields.


# TODO: Déplacer dans widgets.py.
class ImageDroppableHiddenInput(forms.HiddenInput):
    def __init__(self, *args, **kwargs):
        super(ImageDroppableHiddenInput, self).__init__(*args, **kwargs)
        self.related_model, self.related_fieldname = None, None
        self.image_container_html = ''
        self.message = ''

    class Media:
        css = {
            'all': ('django_stachoutils/css/forms.css',)
        }
        js = ('django_stachoutils/js/forms.js',)

    def render(self, name, value, attrs=None):
        hidden_input = super(ImageDroppableHiddenInput, self).render(name, value, attrs=None)
        image_tag = '<img />'
        if value:
            rel_obj = self.related_model.objects.get(pk=value)  # TODO: Faire un get_object_or_none
            image_tag = self._get_thumbnail(rel_obj)

        tag = (
            '<div class="droppableHiddenInput">%s'
            '    <div class="droppableContainer"><span class="delete" title="Vider l\'emplacement"></span>%s'
            '        <div class="droppable"><div class="draggable">%s</div></div>'
            '    </div>'
            '    <div class="message">%s</div>'
            '</div>' % (hidden_input, self.image_container_html, image_tag, self.message)
        )
        return mark_safe(tag)

    def _get_thumbnail(self, rel_obj):
        image_field = getattr(rel_obj, self.related_fieldname)
        if "django_thumbor" in settings.INSTALLED_APPS:
            from django_thumbor import generate_url
            thumbor_server = settings.THUMBOR_SERVER_EXTERNAL
            url = image_field.url
            storage = image_field.storage
            if hasattr(storage, "key"):
                try:
                    url = storage.key(image_field.name)
                # Cas du ThumborMigrationStorage avec image non-migrée sur Thumbor.
                except NotImplementedError:
                    pass
                else:
                    thumbor_server = settings.THUMBOR_SERVER
            return mark_safe('<img src="%s" width="120">' % generate_url(url, thumbor_server=thumbor_server, width=120))
        else:
            from django.template import Context, Template
            t = Template('{% load thumbnail %}{% thumbnail img_field "120" as im %}<img src="{{ im.url }}"'
                         'width="{{ im.width }}" height="{{ im.height }}">{% endthumbnail %}')
            d = {"img_field": image_field}
            return mark_safe(t.render(Context(d)))


class ImageModelChoiceField(forms.ModelChoiceField):
    def __init__(self, related_fieldname, queryset, *args, **kwargs):
        if 'widget' not in kwargs:
            kwargs['widget'] = ImageDroppableHiddenInput
        super(ImageModelChoiceField, self).__init__(queryset, *args, **kwargs)
        self.widget.related_model = queryset.model
        self.widget.related_fieldname = related_fieldname

    def set_image_container_html(self, html):
        self.widget.image_container_html = html

    def set_message(self, html):
        self.widget.message = html



# -*- coding: utf-8 -*-

import hashlib
import re
import string

from datetime import datetime
from django.core.exceptions import ValidationError
from django.utils import formats, dateformat
from django.utils.encoding import force_unicode
from django.utils.functional import allow_lazy
from django.utils.http import urlunquote


def format_datetime(value=None, date=True, time=True):
    if not date:
        format_type = 'TIME_FORMAT'
    elif not time:
        format_type = 'DATE_FORMAT'
    else:
        format_type = 'DATETIME_FORMAT'
    return dateformat.format(value or datetime.now(), formats.get_format(format_type))


def format_number(value=None, separator=','):
    if not value:
        return ''
    def split_thousands(s):
        if len(s) <= 3:
            return s
        return split_thousands(s[:-3]) + separator + s[-3:]
    return split_thousands(str(value))


# From http://stackoverflow.com/questions/517923/what-is-the-best-way-to-remove-accents-in-a-python-unicode-string
import unicodedata
def strip_accents(str):
    #return ''.join([c for c in unicodedata.normalize('NFD', str) if not unicodedata.combining(c)])
    return unicodedata.normalize('NFD', unicode(str)).encode('ascii', 'ignore')


def latin1_safe_xml_encode(str):
    return unicode(str.encode('latin-1', 'xmlcharrefreplace'), 'latin-1')


xml_entity_table = string.maketrans(' \'', '--')
def format_xml_entity(str):
    return strip_accents(str).upper().translate(xml_entity_table)


def truncate_chars(s, num):
    "Truncates a string after a certain number of characters."
    string = force_unicode(s)
    length = int(num)
    if len(string) > length:
        string = string[:length] + '...'
    return string
truncate_chars = allow_lazy(truncate_chars, unicode)


rx_lettrine = re.compile("(\w)(\s)([\w-]+)([,.]{0,1}\s.*)", re.UNICODE|re.IGNORECASE)
def clean_lettrine(txt):
    """Vire les espaces ajoutés aux textes suite import QuarkXpress"""
    match = rx_lettrine.match(txt)
    if match:
        match = match.groups()
        supp_premier_espace = True
        if match[0] == 'A':
            if match[2] not in ('u', 'ux', 'vec', 'utour',):
                supp_premier_espace = False
        if supp_premier_espace:
            txt = rx_lettrine.sub(r'\1\3\4', txt)
    return txt


# From http://www.daniweb.com/code/snippet216865.html
#
# convert an integer to a roman numeral
# keep it reasonable since from 5000 on special characters are used
# see also: http://en.wikipedia.org/wiki/Roman_numerals
# tested with Python24       vegaseat        25jan2007
def int_to_roman(number):
    numerals = { 1 : "I", 4 : "IV", 5 : "V", 9 : "IX", 10 : "X", 40 : "XL",
        50 : "L", 90 : "XC", 100 : "C", 400 : "CD", 500 : "D", 900 : "CM", 1000 : "M" }
    result = ""
    for value, numeral in sorted(numerals.items(), reverse=True):
        while number >= value:
            result += numeral
            number -= value
    return result


# Inspired by http://stackoverflow.com/questions/92438/stripping-non-printable-characters-from-a-string-in-python
non_printable_re = re.compile('[%s]' % re.escape(''.join(map(unichr, range(0, 9) + range(11, 13) + range(14, 32) + range(127, 160)))))
def filter_non_printable(str):
    return non_printable_re.sub('', str)


def simple_decorator(decorator):
    def new_decorator(f):
        g = decorator(f)
        g.__name__ = f.__name__
        g.__doc__ = f.__doc__
        g.__dict__.update(f.__dict__)
        return g
    new_decorator.__name__ = decorator.__name__
    new_decorator.__doc__ = decorator.__doc__
    new_decorator.__dict__.update(decorator.__dict__)
    return new_decorator


def camelize(txt):
    return ' '.join([t.title() for t in txt.split(' ')])


def files_are_equal(f1, f2):
    if f1.size != f2.size:
        return False
    elif _digest_file(f1) != _digest_file(f2):
        return False
    return True


def _digest_file(f, chunk_size=8192):
    md5 = hashlib.md5()
    for chunk in iter(lambda: f.read(chunk_size), ""):
        md5.update(chunk)
    f.seek(0)
    return md5.digest()


def urldecode(url):
    out = {}
    get_params = (url.split('?')[1:] or [None])
    if get_params[0]:
        for param in get_params[0].split('&'):
            k, v = param.split('=')
            out[k] = urlunquote(v)
    return out


NO_CAPFIRST = ['de', 'des', 'du', 'le', 'la', 'les', 'd', 'l', 'a', 'au', 'aux']
def lowerize(txt, excluded=[]):
    words = txt.split(' ')
    cap_words = []
    for i, word in enumerate(words):
        if not word:
            continue
        word = word.lower()
        if not (word in excluded and i>0):
            word = word[0].capitalize() + word[1:]
        cap_words.append(word)
    return " ".join(cap_words)



import os
from distutils.core import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.txt')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name="django-stachoutils",
    version="1.9.5",
    license='BSD Licence',
    author='Stanislas Guerra',
    author_email='stan@slashdev.me',
    description='Commons for Django',
    url='https://github.com/Starou/django-stachoutils',
    long_description=README,
    packages=[
        'django_stachoutils',
        'django_stachoutils.management',
        'django_stachoutils.management.commands',
        'django_stachoutils.forms',
        'django_stachoutils.views',
        'django_stachoutils.templatetags',
    ],
    # I fucking hate distutils-wasting-time-machine.
    # python setup.py build does not give a shit about what is in MANIFEST.in
    # and here in package_data there is now fucking way to use ** glob style so
    # you have to maintain that fucking list.
    # A workaround is to use from setuptools import find_packages and
    # set include_package_data=True but in that case you cannot upload to Pypi!
    # Or maybe the doc sucks. Or maybe I am stupid.
    package_data={
        'django_stachoutils': [
            'static/django_stachoutils/css/*.css',
            'static/django_stachoutils/js/*.js',
            'static/django_stachoutils/img/*.png',
            'static/django_stachoutils/img/*.jpg',
            'static/django_stachoutils/img/*.gif',
            'static/django_stachoutils/poshytip/*.js',
            'static/django_stachoutils/poshytip/*/*.css',
            'static/django_stachoutils/poshytip/*/*.gif',
            'static/django_stachoutils/poshytip/*/*.png',
            'templates/django_stachoutils/*.html',
        ]
    },
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ]
)



# https://gist.github.com/sidmitra/646372
class ExceptionUserInfoMiddleware(object):
    """
    Adds user details to request context on receiving an exception, so that they show up in the error emails.
    Add to settings.MIDDLEWARE_CLASSES and keep it outermost(i.e. on top if possible). This allows
    it to catch exceptions in other middlewares as well.
    """
    
    def process_exception(self, request, exception):
        """
        Process the exception.

        :Parameters:
        - `request`: request that caused the exception
        - `exception`: actual exception being raised
        """
        
        try:
            if request.user.is_authenticated():
                request.META['USERNAME'] = str(request.user.username)
                request.META['USER_EMAIL'] = str(request.user.email)
        except:
            pass
